@@ -1,12 +1,20 @@
 # -*- coding: utf-8 -*-
+import concurrent.futures
 import math
+import os
 import sys
 import unittest
+from unittest.mock import patch
 
+import numpy as np
 import pytest
+from numpy.testing import assert_almost_equal
 
+import pyproj
 from pyproj import Geod, Proj, pj_ellps, pj_list, transform
-from pyproj.crs import CRSError
+from pyproj.exceptions import CRSError, ProjError
+from pyproj.geod import reverse_azimuth
+from test.conftest import proj_network_env
 
 
 class BasicTest(unittest.TestCase):
@@ -31,8 +39,8 @@ class BasicTest(unittest.TestCase):
         )
         awips221_from_dict = Proj(params, preserve_units=False)
 
-        items = sorted([val for val in awips221.crs.srs.split() if val])
-        items_dict = sorted([val for val in awips221_from_dict.crs.srs.split() if val])
+        items = sorted(val for val in awips221.crs.srs.split() if val)
+        items_dict = sorted(val for val in awips221_from_dict.crs.srs.split() if val)
         self.assertEqual(items, items_dict)
 
         expected = sorted(
@@ -117,19 +125,19 @@ class TypeError_Transform_Issue8_Test(unittest.TestCase):
     # https://github.com/jswhit/pyproj/issues/8
 
     def setUp(self):
-        with pytest.warns(DeprecationWarning):
+        with pytest.warns(FutureWarning):
             self.p = Proj(init="epsg:4269")
 
     def test_tranform_none_1st_parmeter(self):
         # test should raise Type error if projections are not of Proj classes
         # version 1.9.4 produced AttributeError, now should raise TypeError
-        with self.assertRaises(CRSError):
+        with pytest.warns(FutureWarning), pytest.raises(CRSError):
             transform(None, self.p, -74, 39)
 
     def test_tranform_none_2nd_parmeter(self):
         # test should raise Type error if projections are not of Proj classes
         # version 1.9.4 has a Segmentation Fault, now should raise TypeError
-        with self.assertRaises(CRSError):
+        with pytest.warns(FutureWarning), pytest.raises(CRSError):
             transform(self.p, None, -74, 39)
 
 
@@ -148,7 +156,8 @@ class ProjLatLongTypeErrorTest(unittest.TestCase):
         p = Proj("+proj=stere +lon_0=-39 +lat_0=90 +lat_ts=71.0 +ellps=WGS84")
         self.assertTrue(isinstance(p, Proj))
         # if not patched this line raises a "TypeError: p2 must be a Proj class"
-        lon, lat = transform(p, p.to_latlong(), 200000, 400000)
+        with pytest.warns(FutureWarning):
+            lon, lat = transform(p, p.to_latlong(), 200000, 400000)
 
 
 @unittest.skipIf(
@@ -212,7 +221,12 @@ class ReprTests(unittest.TestCase):
     # test __repr__ for Proj object
     def test_repr(self):
         p = Proj(proj="latlong", preserve_units=True)
-        expected = "Proj('+proj=longlat +datum=WGS84 +no_defs', preserve_units=True)"
+        expected = (
+            "<Other Coordinate Operation Transformer: longlat>\n"
+            "Description: PROJ-based coordinate operation\n"
+            "Area of Use:\n"
+            "- undefined"
+        )
         self.assertEqual(repr(p), expected)
 
     # test __repr__ for Geod object
@@ -229,7 +243,7 @@ class ReprTests(unittest.TestCase):
             if ellps_name in ("NWL9D", "WGS66"):
                 continue
             p = Geod(ellps=ellps_name)
-            expected = "Geod(ellps='{0}')".format(ellps_name)
+            expected = f"Geod(ellps='{ellps_name}')"
             self.assertEqual(repr(p), expected)
 
 
@@ -247,7 +261,6 @@ class TestRadians(unittest.TestCase):
         )
 
     def test_inv_radians(self):
-
         # Get bearings and distance from Boston to Portland in degrees
         az12_d, az21_d, dist_d = self.g.inv(
             self.boston_d[0],
@@ -287,14 +300,22 @@ class TestRadians(unittest.TestCase):
         )
 
         # Calculate Portland's lon/lat from bearing and distance in radians
-        endlon_r, endlat_r, backaz_r = self.g.fwd(
-            self.boston_r[0], self.boston_r[1], math.radians(az12_d), dist, radians=True
-        )
+        for return_back_azimuth in [False, True]:
+            endlon_r, endlat_r, backaz_r = self.g.fwd(
+                self.boston_r[0],
+                self.boston_r[1],
+                math.radians(az12_d),
+                dist,
+                radians=True,
+                return_back_azimuth=return_back_azimuth,
+            )
+            if not return_back_azimuth:
+                backaz_r = reverse_azimuth(backaz_r, radians=True)
 
-        # Check they are equal
-        self.assertAlmostEqual(endlon_d, math.degrees(endlon_r))
-        self.assertAlmostEqual(endlat_d, math.degrees(endlat_r))
-        self.assertAlmostEqual(backaz_d, math.degrees(backaz_r))
+            # Check they are equal
+            self.assertAlmostEqual(endlon_d, math.degrees(endlon_r))
+            self.assertAlmostEqual(endlat_d, math.degrees(endlat_r))
+            self.assertAlmostEqual(backaz_d, math.degrees(backaz_r))
 
         # Check to make sure we're back in Portland
         self.assertAlmostEqual(endlon_d, self.portland_d[0])
@@ -372,7 +393,8 @@ class Geod_NaN_Issue112_Test(unittest.TestCase):
 def test_proj_equals():
     assert Proj(4326) == Proj("epsg:4326")
     assert Proj(4326) != Proj("epsg:3857")
-    assert Proj(4326) == Proj(Proj("epsg:4326").crs.to_proj4())
+    with pytest.warns(UserWarning):
+        assert Proj(4326) == Proj(Proj("epsg:4326").crs.to_proj4())
 
 
 def test_initialize_proj_crs_no_proj4():
@@ -395,6 +417,12 @@ def test_initialize_proj_crs_no_plus():
     assert proj.crs.srs == "proj=lonlat type=crs"
 
 
+def test_initialize_projparams_with_kwargs():
+    proj_mixed_args = Proj("+proj=utm +zone=10", ellps="WGS84")
+    proj_positional = Proj("+proj=utm +zone=10 +ellps=WGS84")
+    assert proj_mixed_args.is_exact_same(proj_positional)
+
+
 def test_equals_different_type():
     assert Proj("epsg:4326") != ""
 
@@ -411,5 +439,153 @@ def test_reset_errno():
     assert proj(0, 0, inverse=True, errcheck=True) == (0.0, -90.0)
 
 
-if __name__ == "__main__":
-    unittest.main()
+@pytest.mark.parametrize("radians", [False, True])
+def test_get_factors__2d_input(radians):
+    transformer = Proj(3857)
+    longitude = np.array([[0, 1], [2, 3]])
+    latitude = np.array([[1, 2], [3, 4]])
+    if radians:
+        longitude = np.radians(longitude)
+        latitude = np.radians(latitude)
+    factors = transformer.get_factors(
+        longitude=longitude, latitude=latitude, radians=radians
+    )
+    assert_almost_equal(
+        factors.meridional_scale, [[1.0001523, 1.0006095], [1.0013723, 1.0024419]]
+    )
+    assert_almost_equal(
+        factors.parallel_scale, [[1.0001523, 1.0006095], [1.0013723, 1.0024419]]
+    )
+    assert_almost_equal(
+        factors.areal_scale, [[1.00030468, 1.00121946], [1.00274658, 1.00488976]]
+    )
+    assert_almost_equal(factors.angular_distortion, [[0, 0], [0, 0]], decimal=5)
+    assert_almost_equal(
+        factors.meridian_parallel_angle, [[89.99, 90], [90, 90]], decimal=2
+    )
+    assert_almost_equal(factors.meridian_convergence, [[0, 0], [0, 0]])
+    assert_almost_equal(
+        factors.tissot_semimajor, [[1.00015234, 1.00060955], [1.00137235, 1.0024419]]
+    )
+    assert_almost_equal(
+        factors.tissot_semiminor, [[1.00015232, 1.00060953], [1.00137235, 1.0024419]]
+    )
+    assert_almost_equal(factors.dx_dlam, [[1, 1], [1, 1]])
+    assert_almost_equal(factors.dx_dphi, [[0, 0], [0, 0]])
+    assert_almost_equal(factors.dy_dlam, [[0, 0], [0, 0]])
+    assert_almost_equal(
+        factors.dy_dphi, [[1.00015233, 1.00060954], [1.00137235, 1.0024419]]
+    )
+
+
+def test_get_factors():
+    transformer = Proj(3717)
+    factors = transformer.get_factors(-120, 34)
+    assert_almost_equal(factors.meridional_scale, 1.0005466)
+    assert_almost_equal(factors.parallel_scale, 1.0005466)
+    assert_almost_equal(factors.areal_scale, 1.00109349)
+    assert_almost_equal(factors.angular_distortion, 0)
+    assert_almost_equal(factors.meridian_parallel_angle, 90)
+    assert_almost_equal(factors.meridian_convergence, 1.67864770)
+    assert_almost_equal(factors.tissot_semimajor, 1.00055, decimal=5)
+    assert_almost_equal(factors.tissot_semiminor, 1.00055, decimal=5)
+    assert_almost_equal(factors.dx_dlam, 0.8300039)
+    assert_almost_equal(factors.dx_dphi, -0.0292052)
+    assert_almost_equal(factors.dy_dlam, 0.0243244)
+    assert_almost_equal(factors.dy_dphi, 0.9965495)
+
+
+def test_get_factors__nan_inf():
+    transformer = Proj(3857)
+    factors = transformer.get_factors(
+        longitude=[0, np.nan, np.inf, 0], latitude=[np.nan, 2, 2, np.inf]
+    )
+    assert_almost_equal(factors.meridional_scale, [np.inf, np.inf, np.inf, np.inf])
+    assert_almost_equal(factors.parallel_scale, [np.inf, np.inf, np.inf, np.inf])
+    assert_almost_equal(factors.areal_scale, [np.inf, np.inf, np.inf, np.inf])
+    assert_almost_equal(factors.angular_distortion, [np.inf, np.inf, np.inf, np.inf])
+    assert_almost_equal(
+        factors.meridian_parallel_angle, [np.inf, np.inf, np.inf, np.inf]
+    )
+    assert_almost_equal(factors.meridian_convergence, [np.inf, np.inf, np.inf, np.inf])
+    assert_almost_equal(factors.tissot_semimajor, [np.inf, np.inf, np.inf, np.inf])
+    assert_almost_equal(factors.tissot_semiminor, [np.inf, np.inf, np.inf, np.inf])
+    assert_almost_equal(factors.dx_dlam, [np.inf, np.inf, np.inf, np.inf])
+    assert_almost_equal(factors.dx_dphi, [np.inf, np.inf, np.inf, np.inf])
+    assert_almost_equal(factors.dy_dlam, [np.inf, np.inf, np.inf, np.inf])
+    assert_almost_equal(factors.dy_dphi, [np.inf, np.inf, np.inf, np.inf])
+
+
+def test_get_factors__errcheck():
+    transformer = Proj(3857)
+    with pytest.raises(ProjError):
+        transformer.get_factors(longitude=40, latitude=70, errcheck=True, radians=True)
+
+
+def test_numpy_bool_kwarg_false():
+    # Issue 564
+    south = np.array(50) < 0
+    proj = Proj(
+        proj="utm", zone=32, ellipsis="WGS84", datum="WGS84", units="m", south=south
+    )
+    assert "south" not in proj.srs
+
+
+def test_numpy_bool_kwarg_true():
+    # Issue 564
+    south = np.array(50) > 0
+    proj = Proj(
+        proj="utm", zone=32, ellipsis="WGS84", datum="WGS84", units="m", south=south
+    )
+    assert "+south " in proj.srs
+
+
+@patch.dict("os.environ", {"PROJ_NETWORK": "ON"}, clear=True)
+def test_network__disable():
+    with proj_network_env():
+        pyproj.network.set_network_enabled(active=False)
+        transformer = Proj(3857)
+        assert transformer.is_network_enabled is False
+
+
+@patch.dict("os.environ", {"PROJ_NETWORK": "OFF"}, clear=True)
+def test_network__enable():
+    with proj_network_env():
+        pyproj.network.set_network_enabled(active=True)
+        transformer = Proj(3857)
+        assert transformer.is_network_enabled is True
+
+
+def test_network__default():
+    with proj_network_env():
+        pyproj.network.set_network_enabled()
+        transformer = Proj(3857)
+        assert transformer.is_network_enabled == (
+            os.environ.get("PROJ_NETWORK") == "ON"
+        )
+
+
+def test_radians():
+    proj = Proj(
+        {"proj": "lcc", "R": 6371200, "lat_1": 50, "lat_2": 50, "lon_0": -107},
+        preserve_units=False,
+    )
+    assert_almost_equal(
+        proj(math.radians(-145.5), math.radians(1.0), radians=True),
+        (-5632642.22547495, 1636571.4883145525),
+    )
+
+
+@pytest.mark.skipif(
+    pyproj._datadir._USE_GLOBAL_CONTEXT, reason="Global Context not Threadsafe."
+)
+def test_proj_multithread():
+    # https://github.com/pyproj4/pyproj/issues/782
+    trans = Proj("EPSG:3857")
+
+    def transform(num):
+        return trans(1, 2)
+
+    with concurrent.futures.ThreadPoolExecutor(max_workers=2) as executor:
+        for result in executor.map(transform, range(10)):
+            pass
